#include <ros/ros.h>
#include <cv_bridge/cv_bridge.h>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <vector>

class ImageProc {
public:
  
  void imageCb(const sensor_msgs::Image::ConstPtr &msg)  {
    cv_bridge::CvImagePtr cv_ptr =
      cv_bridge::toCvCopy(msg, sensor_msgs::image_encodings::BGR8);

    cv::Mat image = cv_ptr->image;

    ROS_INFO("%ld %ld", (long int)image.rows, (long int)image.cols);

    std::vector<cv::Mat> chans;
    split(cv_ptr->image, chans);

    cv::imshow("Image", image);
    //cv::imshow("B", chans[0]);
    //cv::imshow("G", chans[1]);
    //cv::imshow("R", chans[2]);

    cv::Mat bMinusG;
    cv::Mat bMinusR;
    cv::subtract(chans[0], chans[1], bMinusG);
    cv::subtract(chans[0], chans[2], bMinusR);
    //cv::imshow("bMinusG", bMinusG);
    cv::imshow("bMinusR", bMinusR);

    cv::Mat bThresh;
    cv::threshold(bMinusR, bThresh, 50, 255, cv::THRESH_BINARY);
    cv::imshow("bThresh", bThresh);

    std::vector<cv::Mat> contours;
    std::vector<cv::Vec4i> hierarchy;
    findContours(bThresh, contours, hierarchy, CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE);

    int maxSizeContour = 0;
    int maxContourSize = 0;
    for(int i = 0; i < contours.size(); i++) {
      int contourSize = cv::contourArea(contours[i]);
      if(contourSize > maxContourSize) {
        maxSizeContour = i;
        maxContourSize = contourSize;
      }
    }
    
    
    cv::Mat dst = cv::Mat::zeros(image.rows, image.cols, CV_8UC3);
    drawContours( dst, contours, maxSizeContour, cv::Scalar(255,255,255), cv::LineTypes::FILLED, 8, hierarchy );
    cv::imshow("dst", dst);

    cv::Mat mask = cv::Mat::zeros(image.rows, image.cols, CV_8UC1);
    drawContours( mask, contours, maxSizeContour, cv::Scalar(255), cv::LineTypes::FILLED, 8, hierarchy );

    cv::Mat blueCupImg;
    image.copyTo(blueCupImg, mask);
    cv::imshow("blueCupImg", blueCupImg);

    //There is an OpenCV command
    //bitwise_or(InputArray src1, InputArray src2, OutputArray dst, InputArray mask=noArray())

    //Your homework is to find the red, blue, and green cups, create an output image containing only those cups (you can assume that the largest blob you find is the cup, as long as it basically "is" the cup, we will grade based on how good this looks, but you don't need to pick up the scrap of cup under my hand).
    //Stream this output image on a ROS topic using cv_bridge
    //We should be able to render your topic in rviz
    //Call your topic color_filter/cups


    cv::waitKey(1);
  }

};

int main(int argc, char **argv) {
  ros::init(argc, argv, "mapping_marker");
  ros::NodeHandle node;

  ImageProc ip;
  ros::Subscriber sub = node.subscribe("/kinect2/hd/image_color"
    , 100, &ImageProc::imageCb, &ip);

  ros::spin();
  return 0;
}
